/*
 * File   :     diff.java
 * Author :     Tis Veugen
 * Date   :     1999-04-08
 * Version:     1.0
 */

import java.io.*;
import java.awt.*;
import java.awt.event.*;

public class diff extends Frame
   implements ActionListener, ItemListener, TextListener
{
  private Button                mInput;
  private Button                mOutput;
  private Button                mDiff;

  private TextField             mInputTextField;
  private TextField             mOutputTextField;

  private BufferedInputStream   mInputFile;
  private BufferedOutputStream  mOutputFile;

  private String                mDirectory = "";
  private boolean               mbDirectory;

  public                        diff()
  {
    mInputFile  = null;
    mOutputFile = null;

    setTitle( "Difference (v1.0)" );

    GridBagLayout               lGbl = new GridBagLayout();
    setLayout( lGbl );

    mInput  = addButton( "Dir-1" );
    mOutput = addButton( "Dir-2" );
    mDiff   = addButton( "Diff" );

    mInputTextField  = addTextField();
    mOutputTextField = addTextField();

    GridBagConstraints          lGbc = new GridBagConstraints();

    lGbc.anchor = GridBagConstraints.CENTER;
    lGbc.fill   = GridBagConstraints.HORIZONTAL;
    lGbc.weightx = 20;
    lGbc.weighty = 10;

    add( mInput,   lGbc, 0, 0, 1, 1 );
    add( mOutput,  lGbc, 0, 1, 1, 1 );

    lGbc.fill = GridBagConstraints.NONE;
    add( mInputTextField,  lGbc, 1, 0, 3, 1 );
    add( mOutputTextField, lGbc, 1, 1, 3, 1 );
    add( mDiff, lGbc, 2, 3, 1, 1 );

    addWindowListener( new WindowAdapter()
                       {
                         public void windowClosing(
                         WindowEvent evt )
                         {
                           setVisible( false );
                           System.exit( 0 );
                         }
                       }
                     );

    setLocation( 100, 100 );
    setSize( 400, 100 );
}

  public void                   add(
  Component                     oComponent,
  GridBagConstraints            oGbc,
  int                           x,
  int                           y,
  int                           w,
  int                           h )
  {
    oGbc.gridx = x;
    oGbc.gridy = y;
    oGbc.gridwidth  = w;
    oGbc.gridheight = h;
    add( oComponent, oGbc );
  }

  public Button                 addButton(
  String                        oName )
  {
    Button                      lButton = new Button( oName );

    lButton.addActionListener( this );

    return lButton;
  }

  public TextField              addTextField()
  {
    TextField                   lTextField = new TextField( "", 40 );

    lTextField.addTextListener( this );

    return lTextField;
  }

  public void                   ActionInput()
  {
    String                      lFilename;
    String                      lExtension;

    lFilename = SetFile( "*.*", "Input file", FileDialog.LOAD );
    if ( lFilename != null )
    {
      mInputTextField.setText( lFilename );
      mOutputTextField.setText( lFilename );
    }
  }

  private void                  CloseInputFile()
  {
    if ( mInputFile != null )
    {
      try
      {
        mInputFile.close();
      }
      catch ( Exception e )
      {
        System.err.println( "Can't close Input file" );
      }
    }

    mInputFile = null;
  }

  private String                SetFile(
  String                        oFilter,
  String                        oHeader,
  int                           iMode )
  {
    FileDialog                  lFd;
    String                      lFilename;

    lFd = new FileDialog( this      // mFrame
                        , oHeader
                        , iMode );
    lFd.setDirectory( mDirectory );
    lFd.setFile( oFilter );
    lFd.show();

    lFilename = lFd.getFile();
    if ( lFilename != null )
    {
      mDirectory = lFd.getDirectory();
      lFilename = mDirectory + lFilename;
    }

    return lFilename;
  }

  private void                  PrintDialog(
  boolean                       bDialog,
  Frame                         oFrame,
  String                        oHeader,
  String                        oText )
  {
    if ( bDialog )
    {
      new PbnDialog( oFrame, oHeader, oText );
    }
    else
    {
      System.out.println( oText );
    }
  }

  private int                   FileInList(
  File                          oDir1File,
  File []                       oDir2Files )
  {
    int                         nDir2 = oDir2Files.length;
    int                         iDir2;
    String                      lFilename = oDir1File.getName();

    for ( iDir2 = 0 ; iDir2 < nDir2; iDir2++ )
    {
      if ( lFilename.equals( oDir2Files[ iDir2 ].getName() ) )
      {
        return iDir2;
      }
    }

    return -1;
  }        

  private long                  EqualFiles(
  File                          oFile1,
  File                          oFile2 )
  {
    long                        n1;
    long                        n2;
    FileInputStream             lFis1;
    BufferedInputStream         lBis1;
    DataInputStream             lDis1;
    FileInputStream             lFis2;
    BufferedInputStream         lBis2;
    DataInputStream             lDis2;

    if ( oFile1.equals( oFile2 ) )
    {
      return -1;
    }

    try
    {
      n1 = oFile1.length();
      n2 = oFile2.length();

      lFis1 = new FileInputStream( oFile1 );
      lBis1 = new BufferedInputStream( lFis1 );
      lDis1 = new DataInputStream( lBis1 );

      lFis2 = new FileInputStream( oFile2 );
      lBis2 = new BufferedInputStream( lFis2 );
      lDis2 = new DataInputStream( lBis2 );
    }
    catch ( Exception e )
    {
      System.err.println( "Can't compare files" );
      return -1;
    }

    long                     bOk = -1;

    for ( long i = 0; i < n1; i++ )
    {
      try
      {
        if ( lDis1.read() != lDis2.read() )
        {
          bOk = i;
        }
      }
      catch ( Exception e )
      {
        System.err.println( "Can't get read Files" );
        bOk = 0;
      }

      if ( bOk >= 0 )
      {
        break;
      }
    }

    try
    {
      lFis1.close();
      lFis2.close();
    }
    catch ( Exception e )
    {
      System.err.println( "Can't close files" );
    }

    return bOk;
  }

  private boolean               ActionConvert(
  File []                       oDir1Files,
  File []                       oDir2Files,
  boolean                       bDialog )
  {
    int                         nDir1 = oDir1Files.length;
    int                         nDir2 = oDir2Files.length;
    int                         iDir1;
    int                         iDir2;
    boolean []                  laDir2Used = new boolean[ nDir2 ];

    if ( bDialog )
    {
      this.setCursor( WAIT_CURSOR );
    }

    for ( iDir2 = 0 ; iDir2 < nDir2; iDir2++ )
    {
      laDir2Used[ iDir2 ] = false;
    }

    for ( iDir1 = 0 ; iDir1 < nDir1; iDir1++ )
    {
      iDir2 = FileInList( oDir1Files[ iDir1 ], oDir2Files );
      if ( iDir2 < 0 )
      {
        PrintDialog( bDialog, this, "NOT in 2"
                   , oDir1Files[ iDir1 ].getName() );
      }
      else
      {
        laDir2Used[ iDir2 ] = true;
        long d = EqualFiles( oDir1Files[ iDir1 ], oDir2Files[ iDir2 ] );
        if ( d >= 0 )
        {
          PrintDialog( bDialog, this, "DIFFER@ " + d
                     , oDir1Files[ iDir1 ].getName() );
        }
      }
    }

    if ( ! mbDirectory )
    {
      for ( iDir2 = 0 ; iDir2 < nDir2; iDir2++ )
      {
        if ( ! laDir2Used[ iDir2 ] )
        {
          PrintDialog( bDialog, this, "NOT in 1"
                     , oDir2Files[ iDir2 ].getName() );
        }
      }
    }

    if ( bDialog )
    {
      this.setCursor( DEFAULT_CURSOR );
    }

    return true;
  }

  private File []               GetAllFiles(
  String                        oFilename )
  {
    File   []                   lFiles = null;
    String                      lDirectory = "";
    String []                   lFilenames;
    File                        lFile = new File( oFilename );

    if ( ( oFilename.indexOf( "*" ) == -1 )
      && ( oFilename.indexOf( "?" ) == -1 ) )
    { /*
       * Check single file or directory.
       */
      if ( lFile.isFile() )
      {
        lFilenames = new String[ 1 ];
        lFilenames[ 0 ] = oFilename;
        lDirectory = "";
      }
      else
      { /*
         * A directory.
         */
        lFilenames = lFile.list();
        lDirectory = oFilename;
        mbDirectory = true;
      }
    }
    else
    { /*
       * If a file has wildcards, then it is not a directory.
       */
      lDirectory = lFile.getParent();
      File              lPath = new File( lDirectory );
      lFilenames = lPath.list();
      if ( lFilenames != null )
      {
        lFilenames = PatternMatch.MatchList( lFilenames
                                           , lFile.getName() );
      }
    }

    if ( lFilenames != null )
    {
      int                       n = lFilenames.length;

      lFiles = new File[ n ];
      for ( int i = 0; i < n; i++ )
      {
        lFiles[ i ] = new File( lDirectory, lFilenames[ i ] );
      }
    }

    return lFiles;
  }

  public void                   actionPerformed(
  ActionEvent                   evt )
  {
    String                      lName = evt.getActionCommand();
    String                      lFilename;

    if ( lName.equals( "Dir-1" ) )
    {
      ActionInput();
    }
    else
    if ( lName.equals( "Dir-2" ) )
    {
      lFilename = SetFile( "*.*", "Output file", FileDialog.LOAD );
      if ( lFilename != null )
      {
        mOutputTextField.setText( lFilename );
      }
    }
    else
    if ( lName.equals( "Diff" ) )
    {
      File []                   lDir1Files;
      File []                   lDir2Files;

      lDir1Files = GetAllFiles( mInputTextField.getText() );
      /*
       * It is important to know whether Dir2 is a directory for reporting absent files.
       */
      mbDirectory = false;
      lDir2Files = GetAllFiles( mOutputTextField.getText() );
      if ( lDir1Files == null )
      {
        PrintDialog( true, this, "File error", "Can't read Dir1 files" );
      }
      else
      if ( lDir2Files == null )
      {
        PrintDialog( true, this, "File error", "Can't read Dir2 files" );
      }
      else
      {
        ActionConvert( lDir1Files
                     , lDir2Files
                     , true );
        PrintDialog( true, this, "Conversion Result", "Ready" );
      }
    }
  }

  public void                   itemStateChanged(
  ItemEvent                     evt )
  {
    if ( evt.getStateChange() == ItemEvent.DESELECTED )
    {
      return;
    }
  }

  public void                   textValueChanged(
  TextEvent                     evt )
  {
  }

  public static void            main(
  String []                     args )
  {
    Frame                       lDiff;

    lDiff = new diff();
    lDiff.show();
  }
}

class PbnDialog extends Dialog
{
  public                        PbnDialog(
  Frame                         oParent,
  String                        oHeader,
  String                        oText )
  {
    super( oParent, oHeader, true );

    Panel                       lPanel = new Panel();
    int                         NrLines;
    int                         Index;

    for ( NrLines = 0; ; NrLines++ )
    {
      Index = oText.indexOf( "\n" );
      if ( Index < 0 )
      {
        lPanel.add( new Label( oText ) );
        break;
      }
      lPanel.add( new Label( oText.substring( 0, Index ) ) );
      oText = oText.substring( Index+1 );
    }
    add( lPanel, "Center" );

    Button                      mOk = new Button( "OK" );
    lPanel = new Panel();
    lPanel.add( mOk );
    add( lPanel, "South" );

    mOk.addActionListener( new ActionListener()
                           {
                             public void actionPerformed(
                             ActionEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

        addWindowListener( new WindowAdapter()
                           {
                             public void windowClosing(
                             WindowEvent evt )
                             {
                               setVisible( false );
                             }
                           }
                         );

    setLocation( 150, 150 );
    setSize( 300, 120 + NrLines * 30 );
    show();
  }
}
